@@ -146,7 +146,7 @@ class FailManager:
 					del self.__failList[fid]
 			else:
 				# create new dictionary without items to be deleted:
-				self.__failList = dict((fid,item) for fid,item in self.__failList.items() \
+				self.__failList = dict((fid,item) for fid,item in iter(self.__failList.items()) \
 					if item.getLastTime() + self.__maxTime > time)
 		self.__bgSvc.service()
 	
