@@ -300,7 +300,7 @@ class JailReaderTest(LogCaptureTestCase):
 		# And multiple groups (`][` instead of `,`)
 		result = extractOptions(option.replace(',', ']['))
 		expected2 = (expected[0],
-		 dict((k, v.replace(',', '][')) for k, v in expected[1].items())
+		 dict((k, v.replace(',', '][')) for k, v in iter(expected[1].items()))
 		)
 		self.assertEqual(expected2, result)
 
