@@ -452,7 +452,10 @@ class Fail2banClientTest(Fail2banClientServerBase):
 		self.assertLogged("Usage: " + CLIENT)
 		self.assertLogged("Report bugs to ")
 		self.pruneLog()
-		self.execCmd(SUCCESS, (), "-vq", "-V")
+		self.execCmd(SUCCESS, (), "-V")
+		self.assertLogged(fail2bancmdline.normVersion())
+		self.pruneLog()
+		self.execCmd(SUCCESS, (), "-vq", "--version")
 		self.assertLogged("Fail2Ban v" + fail2bancmdline.version)
 		self.pruneLog()
 		self.execCmd(SUCCESS, (), "--str2sec", "1d12h30m")
@@ -578,7 +581,7 @@ class Fail2banClientTest(Fail2banClientServerBase):
 			# test reload missing jail (direct):
 			self.execCmd(FAILED, startparams, "reload", "~~unknown~jail~fail~~")
 			self.assertLogged("Failed during configuration: No section: '~~unknown~jail~fail~~'")
-			self.assertLogged("Exit with code -1")
+			self.assertLogged("Exit with code 255")
 			self.pruneLog()
 		finally:
 			self.pruneLog()
@@ -802,7 +805,7 @@ class Fail2banServerTest(Fail2banClientServerBase):
 				"norestored = %(_exec_once)s",
 				"restore = ",
 				"info = ",
-				"_use_flush_ = echo [<name>] <actname>: -- flushing IPs",
+				"_use_flush_ = echo '[%(name)s] %(actname)s: -- flushing IPs'",
 				"actionstart =  echo '[%(name)s] %(actname)s: ** start'", start,
 				"actionreload = echo '[%(name)s] %(actname)s: .. reload'", reload,
 				"actionban =    echo '[%(name)s] %(actname)s: ++ ban <ip> %(restore)s%(info)s'", ban,
@@ -900,7 +903,7 @@ class Fail2banServerTest(Fail2banClientServerBase):
 		if unittest.F2B.log_level < logging.DEBUG: # pragma: no cover
 			_out_file(test1log)
 		self.execCmd(SUCCESS, startparams, "reload")
-		self.assertLogged("Reload finished.", all=True, wait=MID_WAITTIME)
+		self.assertLogged("Reload finished.", wait=MID_WAITTIME)
 		# test not unbanned / banned again:
 		self.assertNotLogged(
 			"[test-jail1] Unban 192.0.2.1", 
@@ -932,7 +935,7 @@ class Fail2banServerTest(Fail2banClientServerBase):
 			reload="               echo '[<name>] %s: reloaded.'" % "test-action1", 
 			stop=  "               echo '[<name>] %s: stopped.'" % "test-action1")
 		self.execCmd(SUCCESS, startparams, "reload")
-		self.assertLogged("Reload finished.", all=True, wait=MID_WAITTIME)
+		self.assertLogged("Reload finished.", wait=MID_WAITTIME)
 		# test not unbanned / banned again:
 		self.assertNotLogged(
 			"[test-jail1] Unban 192.0.2.1", 
@@ -1064,8 +1067,7 @@ class Fail2banServerTest(Fail2banClientServerBase):
 		# reload jail1 without restart (without ban/unban):
 		self.pruneLog("[test-phase 3]")
 		self.execCmd(SUCCESS, startparams, "reload", "test-jail1")
-		self.assertLogged(
-			"Reload finished.", all=True, wait=MID_WAITTIME)
+		self.assertLogged("Reload finished.", wait=MID_WAITTIME)
 		self.assertLogged(
 			"Reload jail 'test-jail1'",
 			"Jail 'test-jail1' reloaded", all=True)
@@ -1079,7 +1081,7 @@ class Fail2banServerTest(Fail2banClientServerBase):
 		self.pruneLog("[test-phase 4]")
 		_write_jail_cfg(enabled=[1])
 		self.execCmd(SUCCESS, startparams, "reload")
-		self.assertLogged("Reload finished.", all=True, wait=MID_WAITTIME)
+		self.assertLogged("Reload finished.", wait=MID_WAITTIME)
 		# test both jails should be reloaded:
 		self.assertLogged(
 			"Reload jail 'test-jail1'")
@@ -1126,7 +1128,7 @@ class Fail2banServerTest(Fail2banClientServerBase):
 		self.pruneLog("[test-phase 7]")
 		self.execCmd(SUCCESS, startparams,
 			"reload", "--unban")
-		self.assertLogged("Reload finished.", all=True, wait=MID_WAITTIME)
+		self.assertLogged("Reload finished.", wait=MID_WAITTIME)
 		# reloads unbanned all:
 		self.assertLogged(
 			"Jail 'test-jail1' reloaded",
@@ -1157,7 +1159,7 @@ class Fail2banServerTest(Fail2banClientServerBase):
 		self.pruneLog("[test-phase 8a]")
 		_write_jail_cfg(enabled=[1], backend="xxx-unknown-backend-zzz")
 		self.execCmd(FAILED, startparams, "reload")
-		self.assertLogged("Reload finished.", all=True, wait=MID_WAITTIME)
+		self.assertLogged("Reload finished.", wait=MID_WAITTIME)
 		self.assertLogged(
 			"Restart jail 'test-jail1' (reason: 'polling' != ", 
 			"Unknown backend ", all=True)
@@ -1165,18 +1167,20 @@ class Fail2banServerTest(Fail2banClientServerBase):
 		self.pruneLog("[test-phase 8b]")
 		_write_jail_cfg(enabled=[1])
 		self.execCmd(SUCCESS, startparams, "reload")
-		self.assertLogged("Reload finished.", all=True, wait=MID_WAITTIME)	
+		self.assertLogged("Reload finished.", wait=MID_WAITTIME)
 
 		# several small cases (cover several parts):
 		self.pruneLog("[test-phase end-1]")
 		# wrong jail (not-started):
 		self.execCmd(FAILED, startparams,
 			"--async", "reload", "test-jail2")
+		self.assertLogged("Reload finished.", wait=MID_WAITTIME)
 		self.assertLogged("the jail 'test-jail2' does not exist")
 		self.pruneLog()
 		# unavailable jail (but exit 0), using --if-exists option:
 		self.execCmd(SUCCESS, startparams,
 			"--async", "reload", "--if-exists", "test-jail2")
+		self.assertLogged("Reload finished.", wait=MID_WAITTIME)
 		self.assertNotLogged(
 			"Creating new jail 'test-jail2'",
 			"Jail 'test-jail2' started", all=True)
@@ -1185,9 +1189,10 @@ class Fail2banServerTest(Fail2banClientServerBase):
 		self.pruneLog("[test-phase end-2]")
 		self.execCmd(SUCCESS, startparams,
 			"--async", "reload", "--restart", "--all")
+		self.assertLogged("Reload finished.", wait=MID_WAITTIME)
 		self.assertLogged(
 			"Jail 'test-jail1' stopped", 
-			"Jail 'test-jail1' started", all=True)
+			"Jail 'test-jail1' started", all=True, wait=MID_WAITTIME)
 
 	# test action.d/nginx-block-map.conf --
 	@unittest.F2B.skip_if_cfg_missing(action="nginx-block-map")
@@ -1220,8 +1225,8 @@ class Fail2banServerTest(Fail2banClientServerBase):
 		_write_file(lgfn, "w+",
 			str(int(MyTime.time())) + ' failure "125-000-001" - 192.0.2.1',
 			str(int(MyTime.time())) + ' failure "125-000-002" - 192.0.2.1',
-			str(int(MyTime.time())) + ' failure "125-000-003" - 192.0.2.1',
-			str(int(MyTime.time())) + ' failure "125-000-004" - 192.0.2.1',
+			str(int(MyTime.time())) + ' failure "125-000-003" - 192.0.2.1 (\xf2\xf0\xe5\xf2\xe8\xe9)',
+			str(int(MyTime.time())) + ' failure "125-000-004" - 192.0.2.1 (\xf2\xf0\xe5\xf2\xe8\xe9)',
 			str(int(MyTime.time())) + ' failure "125-000-005" - 192.0.2.1',
 		)
 		# check all sessions are banned (and blacklisted in map-file):
